#
# This is the SPEC file for creating binary RPMs for the Dom0.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2013  Marek Marczykowski  <marmarek@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#

%{!?python_sitearch: %define python_sitearch %(%{__python} -c "from distutils.sysconfig import get_python_lib; print get_python_lib(1)")}

%define _dracutmoddir	/usr/lib/dracut/modules.d

Name:		qubes-core-dom0-linux
Version:	4.1.9
Release:	1%{?dist}
Summary:	Linux-specific files for Qubes dom0

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		http://www.qubes-os.org

BuildRequires:  ImageMagick
BuildRequires:  pandoc
BuildRequires:  qubes-utils-devel >= 3.1.3
BuildRequires:  gcc
Requires:	qubes-core-dom0
Requires:	python3-qubesadmin
Requires:	qubes-core-qrexec-dom0
Requires:	qubes-core-admin-client
Requires:	qubes-utils >= 3.1.3
Requires:	qubes-utils-libs >= 4.0.16
Conflicts:	qubes-core-dom0 < 4.0.23
Requires:	%{name}-kernel-install
Requires:	xdotool
Requires:	createrepo_c
# salt needs to pass --console --show-output options now
Conflicts:  qubes-mgmt-salt-dom0-update < 4.1.4

Source0: %{name}-%{version}.tar.gz

%description
Linux customizations required to use system as Qubes dom0.
Additionally some graphical elements for every Linux desktop envirnment (icons,
appmenus etc).

%package kernel-install
Summary:	Kernel install hook for Xen-based system

# get rid of os-prober, it tries to mount and parse all the block devices in
# the system, including loop*
Provides: os-prober
Obsoletes: os-prober

%description kernel-install
Kernel install hook for Xen-based system.

%package -n qubes-core-dom0-vaio-fixes
Summary: Additional scripts for supporting suspend on Vaio Z laptops

%description -n qubes-core-dom0-vaio-fixes
Additional scripts for supporting suspend on Vaio Z laptops.

Due to broken Linux GPU drivers we need to do some additional actions during
suspend/resume.

%prep
%setup -q

%build
export BACKEND_VMM=xen
(cd dom0-updates; make)
(cd file-copy-vm; make)
(cd doc; make manpages)

%install

## Appmenus
install -d $RPM_BUILD_ROOT/etc/qubes-rpc/policy
cp qubesappmenus/qubes.SyncAppMenus.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.SyncAppMenus

### Dom0 updates
install -D dom0-updates/qubes-dom0-updates.cron $RPM_BUILD_ROOT/etc/cron.daily/qubes-dom0-updates.cron
install -D dom0-updates/qubes-dom0-update $RPM_BUILD_ROOT/usr/bin/qubes-dom0-update
install -D dom0-updates/qubes-receive-updates $RPM_BUILD_ROOT/usr/libexec/qubes/qubes-receive-updates
install -D dom0-updates/patch-dnf-yum-config $RPM_BUILD_ROOT/usr/lib/qubes/patch-dnf-yum-config
install -m 0644 -D dom0-updates/qubes-cached.repo $RPM_BUILD_ROOT/etc/yum.real.repos.d/qubes-cached.repo
install -D dom0-updates/qfile-dom0-unpacker $RPM_BUILD_ROOT/usr/libexec/qubes/qfile-dom0-unpacker
ln -s ../../usr/libexec/qubes/qubes-receive-updates $RPM_BUILD_ROOT/etc/qubes-rpc/qubes.ReceiveUpdates
install -m 0664 -D dom0-updates/qubes.ReceiveUpdates.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.ReceiveUpdates

install -d $RPM_BUILD_ROOT/var/lib/qubes/updates

# Qrexec services
mkdir -p $RPM_BUILD_ROOT/usr/lib/qubes/qubes-rpc $RPM_BUILD_ROOT/etc/qubes-rpc/policy
cp qubes-rpc/* $RPM_BUILD_ROOT/usr/lib/qubes/qubes-rpc/
for i in qubes-rpc/*; do ln -s ../../usr/lib/qubes/$i $RPM_BUILD_ROOT/etc/qubes-rpc/$(basename $i); done
cp qubes-rpc-policy/* $RPM_BUILD_ROOT/etc/qubes-rpc/policy/

### pm-utils
mkdir -p $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d
cp pm-utils/52qubes-pause-vms $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d/
mkdir -p $RPM_BUILD_ROOT/usr/lib/systemd/system
cp pm-utils/qubes-suspend.service $RPM_BUILD_ROOT/usr/lib/systemd/system/

### Dracut module
mkdir -p $RPM_BUILD_ROOT/etc/dracut.conf.d
cp dracut/dracut.conf.d/* $RPM_BUILD_ROOT/etc/dracut.conf.d/

mkdir -p $RPM_BUILD_ROOT%{_dracutmoddir}
cp -r dracut/modules.d/* $RPM_BUILD_ROOT%{_dracutmoddir}/

### Others
mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
install -m 0644 -D system-config/limits-qubes.conf $RPM_BUILD_ROOT/etc/security/limits.d/99-qubes.conf
install -D system-config/cpufreq-xen.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/cpufreq-xen.modules
install -m 0440 -D system-config/qubes.sudoers $RPM_BUILD_ROOT/etc/sudoers.d/qubes
install -D system-config/polkit-1-qubes-allow-all.rules $RPM_BUILD_ROOT/etc/polkit-1/rules.d/00-qubes-allow-all.rules
install -D system-config/qubes-dom0.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/qubes-dom0.modules
install -D system-config/qubes-sync-clock.cron $RPM_BUILD_ROOT/etc/cron.d/qubes-sync-clock.cron
install -D system-config/lvm-cleanup.cron-daily $RPM_BUILD_ROOT/etc/cron.daily/lvm-cleanup
install -d $RPM_BUILD_ROOT/etc/udev/rules.d
install -m 644 system-config/00-qubes-ignore-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 system-config/12-qubes-ignore-lvm-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 -D system-config/disable-lesspipe.sh $RPM_BUILD_ROOT/etc/profile.d/zz-disable-lesspipe.sh
install -m 755 -D system-config/kernel-grub2.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/80-grub2.install
install -m 755 -D system-config/kernel-xen-efi.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/90-xen-efi.install
install -m 755 -D system-config/kernel-remove-bls.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/99-remove-bls.install
install -m 644 -D system-config/75-qubes-dom0.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/75-qubes-dom0.preset
install -m 644 -D system-config/75-qubes-dom0-user.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/user-preset/75-qubes-dom0-user.preset
install -m 644 -D system-config/99-qubes-default-disable.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
install -d $RPM_BUILD_ROOT/etc/dnf/protected.d
install -m 0644 system-config/dnf-protected-qubes-core-dom0.conf  \
        $RPM_BUILD_ROOT/etc/dnf/protected.d/qubes-core-dom0.conf


touch $RPM_BUILD_ROOT/var/lib/qubes/.qubes-exclude-block-devices

# file copy to VM
install -m 755 file-copy-vm/qfile-dom0-agent $RPM_BUILD_ROOT/usr/lib/qubes/
install -m 755 file-copy-vm/qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/
install -m 755 file-copy-vm/qvm-copy $RPM_BUILD_ROOT/usr/bin/
ln -s qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/qvm-move-to-vm
ln -s qvm-copy $RPM_BUILD_ROOT/usr/bin/qvm-move

### Documentation
(cd doc; make DESTDIR=$RPM_BUILD_ROOT install)

# Vaio fixes
mkdir -p $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d
install -D vaio-fixes/00sony-vaio-audio $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d/
install -D vaio-fixes/99sony-vaio-audio $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d/
mkdir -p $RPM_BUILD_ROOT/etc/modprobe.d/
install -D vaio-fixes/snd-hda-intel-sony-vaio.conf $RPM_BUILD_ROOT/etc/modprobe.d/

%pre
if ! grep -q ^qubes: /etc/group ; then
		groupadd qubes
fi

%post

/usr/lib/qubes/patch-dnf-yum-config

systemctl enable qubes-suspend.service >/dev/null 2>&1

# migrate dom0-updates check disable flag
if [ $1 -ge 2 ]; then
    if [ -e /var/lib/qubes/updates/disable-updates ]; then
        qvm-features dom0 service.qubes-update-check ''
        rm -f /var/lib/qubes/updates/disable-updates
    fi
fi

%preun
if [ "$1" = 0 ] ; then
	# no more packages left

    systemctl disable qubes-suspend.service > /dev/null 2>&1
fi

%triggerin -- PackageKit
# dom0 have no network, but still can receive updates (qubes-dom0-update)
sed -i 's/^UseNetworkHeuristic=.*/UseNetworkHeuristic=false/' /etc/PackageKit/PackageKit.conf

%triggerin -- xorg-x11-drv-vmmouse
# Remove unnecessary udev rules that causes problems in dom0 (#605)
rm -f /lib/udev/rules.d/69-xorg-vmmouse.rules

%triggerin -- grub2-tools
chmod -x /etc/grub.d/10_linux

%files
/etc/qubes-rpc/policy/qubes.SyncAppMenus
# Dom0 updates
/etc/cron.daily/qubes-dom0-updates.cron
/etc/yum.real.repos.d/qubes-cached.repo
/usr/bin/qubes-dom0-update
/usr/lib/qubes/patch-dnf-yum-config
%attr(4750,root,qubes) /usr/libexec/qubes/qfile-dom0-unpacker
/usr/libexec/qubes/qubes-receive-updates
/etc/qubes-rpc/qubes.ReceiveUpdates
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.ReceiveUpdates
%attr(0770,root,qubes) %dir /var/lib/qubes/updates
# Qrexec services
/etc/qubes-rpc/qubes.repos.*
/usr/lib/qubes/qubes-rpc/qubes.repos.*
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.List
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.Enable
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.Disable
# Dracut module
/etc/dracut.conf.d/*
%dir %{_dracutmoddir}/90macbook12-spi-driver
%{_dracutmoddir}/90macbook12-spi-driver/*
%dir %{_dracutmoddir}/90qubes-pciback
%{_dracutmoddir}/90qubes-pciback/*
%dir %{_dracutmoddir}/90extra-modules
%{_dracutmoddir}/90extra-modules/*
# file copy
/usr/bin/qvm-copy-to-vm
/usr/bin/qvm-move-to-vm
/usr/bin/qvm-copy
/usr/bin/qvm-move
/usr/lib/qubes/qfile-dom0-agent
# pm-utils
/usr/lib64/pm-utils/sleep.d/52qubes-pause-vms
/usr/lib/systemd/system/qubes-suspend.service
# Others
/etc/sysconfig/modules/qubes-dom0.modules
/etc/sysconfig/modules/cpufreq-xen.modules
/etc/sudoers.d/qubes
/etc/polkit-1/rules.d/00-qubes-allow-all.rules
/etc/security/limits.d/99-qubes.conf
%config /etc/udev/rules.d/00-qubes-ignore-devices.rules
%config /etc/udev/rules.d/12-qubes-ignore-lvm-devices.rules
%attr(0644,root,root) /etc/cron.d/qubes-sync-clock.cron
/etc/cron.daily/lvm-cleanup
%config(noreplace) /etc/profile.d/zz-disable-lesspipe.sh
%config(noreplace) /etc/dnf/protected.d/qubes-core-dom0.conf
/usr/lib/systemd/system-preset/75-qubes-dom0.preset
/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
/usr/lib/systemd/user-preset/75-qubes-dom0-user.preset
/var/lib/qubes/.qubes-exclude-block-devices
# Man
%{_mandir}/man1/qvm-*.1*
%{_mandir}/man1/qubes-*.1*

%files kernel-install
/usr/lib/kernel/install.d/80-grub2.install
/usr/lib/kernel/install.d/90-xen-efi.install
/usr/lib/kernel/install.d/99-remove-bls.install

%files -n qubes-core-dom0-vaio-fixes
/usr/lib64/pm-utils/sleep.d/00sony-vaio-audio
/usr/lib64/pm-utils/sleep.d/99sony-vaio-audio
/etc/modprobe.d/snd-hda-intel-sony-vaio.conf

%changelog
* Sun Jan 24 2021 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-core-admin-linux.git

* Sun Jan 24 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - db813ca
- version 4.1.9

* Sun Jan 24 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - bf7bc5d
- rpm: don't update if qubes-mgmt-salt-dom0-update isn't new enough

* Sun Jan 24 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 68d73f1
- qubes-dom0-update: filter what options are sent to the VM

* Thu Jan 21 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 9681485
- Use ‘--quiet’ for ‘qvm-run’

* Thu Jan 21 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 5163b61
- Avoid using ‘exec script’

* Thu Jan 21 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 82da122
- Allow removing TemplateVM packages

* Thu Jan 21 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - ec01b9d
- Show output for non-interactive remote commands

* Sun Jan 03 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4d6fc9b
- qfile-dom0-unpacker: use spaces to indent

* Fri Jan 01 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 3ef780c
- Remove old RPM database

* Thu Dec 31 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b74d660
- qfile-dom0-unpacker: support waiting for free space

* Tue Dec 29 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - d061672
- Imply --console if DISPLAY is unset

* Tue Dec 29 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - e33d6a5
- Require --show-output to display UpdateVM output

* Tue Dec 29 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - b8eb7a4
- Pass GPG keys to UpdateVM

* Thu Dec 17 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - c05b957
- Make -e happy

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 4e869d3
- Add ‘--console’ option for disabling in-VM xterm

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 6e80ee6
- Spawn a VM xterm for the GUI updater

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - dc0c42e
- Docuemnt that --enablerepo foo is not supported

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 786c35e
- Ignore trailing --

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - d5c5212
- Indicate that -- marks end of options

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 542291f
- Don’t use --nogui when running xterm

* Wed Dec 16 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 3a41bbb
- Avoid displaying UpdateVM stderr in dom0

* Fri Dec 11 2020 Demi Marie Obenour <athena@invisiblethingslab.com> - 06b07c1
- qubes-dom0-update: handle -- and metacharacters

* Fri Nov 20 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e1fee06
- version 4.1.8

* Fri Nov 20 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 30c8e2d
- kernel-install: generate initramfs before switching default kernel

* Sun Nov 15 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 527a3b7
- Add .gitlab-ci.yml

* Fri Nov 13 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 31efb7a
- dom0-updates: use 'incoming_dir' for getting freespace

* Wed Nov 11 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 6a8a4e8
- Fix from Marek's comments

* Tue Nov 10 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 02ed544
- dom0-updates: double default max updates bytes

* Sun Nov 08 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1c74dd0
- qubes-dom0-update: fix major xen upgrade protection

* Sun Aug 23 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 7077f73
- version 4.1.7

* Sat Aug 22 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 418f50c
- qubes-dom0-update: better handle major Xen upgrade

* Wed Jun 24 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 0e3870f
- qubes-dom0-update: fix missing '&&'

* Sun Jun 21 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - b63a1fe
- Add vaio-fixes as subpackage

* Sun Jun 07 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 069e2a3
- version 4.1.6

* Sat Jun 06 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - b95f271
- Prevent failing if cannot set 'update-available' feature

* Fri Jun 05 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - df3f29d
- Make ShellCheck happy and fix constant expression

* Sat May 23 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 31173e5
- Fix typo in qubes-receive-updates

* Wed Mar 11 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 75c4156
- version 4.1.5

* Tue Mar 10 2020 hal18000 <hal18000@protonmail.com> - 1a28a1c
- Fix kernel install script clobbering kernel params ending in default=

* Thu Feb 27 2020 Andrey Arapov <andrey.arapov@nixaid.com> - 2df917e
- dracut: add roadrunner2/macbook12-spi-driver on MacBook HW

* Tue Feb 25 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b6f9f2c
- version 4.1.4

* Mon Feb 24 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e59c6b8
- Replace /etc/qubes-rpc/qubes.ReceiveUpdates with a symlink

* Fri Feb 14 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - e4d73bb
- qubes-dom0-update: don't use 'script' if not in interactive mode

* Fri Feb 14 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - e3887aa
- qubes-dom0-update: support 'list' and 'search' actions

* Thu Jan 30 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f98fb82
- dracut: install cirrus and bochs_drm into initramfs on relevant machines

* Wed Jan 15 2020 3hhh <tripleh@hackingthe.net> - c600b1b
- Proper argument escaping for special characters.

* Wed Jan 15 2020 3hhh <tripleh@hackingthe.net> - f4f5731
- Don't pass stdin to VMs unless necessary.

* Sat Jan 11 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 0d53697
- travis: switch to dom0 Fedora 31

* Sat Jan 04 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - a4006f5
- version 4.1.3

* Thu Dec 19 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4a88c52
- kernel-install: consider both grub2 and grub2-efi configs

